/****************************************************************************

    Multimedia Systems Software Development Kit Include File
 
    Copyright (c) Microsoft Corporation 1990. All rights reserved
 
 
    If defined, the following flags inhibit inclusion of the indicated items:
 
        MMNOSOUND        - Sound support
        MMNOWAVE         - Waveform support
        MMNOMIDI         - MIDI support
        MMNOAUX          - Auxiliary output support
        MMNOTIMER        - Timer support
        MMNOJOY          - Joystick support
        MMNOMCI          - MCI support
        MMNODRV          - Installable driver support 
	MMNOMMIO	 - MMIO library support
        MMNOMMSYSTEM     - No mmsystem general functions


****************************************************************************/

#define MMSYSTEM
typedef WORD	VERSION;

/****************************************************************************

    Multimedia Systems Window Messages

****************************************************************************/

#ifndef WM_MM_RESERVED_FIRST
// taken from windows.h
/* The following message range reserved for multi-media */
#define WM_MM_RESERVED_FIRST 0x03B0
#define WM_MM_RESERVED_LAST  0x03EF
#endif

#define MM_JOY1MOVE	    (WM_MM_RESERVED_FIRST + 1)
#define MM_JOY2MOVE	    (1 + MM_JOY1MOVE)
#define MM_JOY1ZMOVE	    (WM_MM_RESERVED_FIRST + 3)
#define MM_JOY2ZMOVE	    (1 + MM_JOY1ZMOVE)
#define MM_JOY1BUTTONDOWN   (WM_MM_RESERVED_FIRST + 5)
#define MM_JOY2BUTTONDOWN   (1 + MM_JOY1BUTTONDOWN)
#define MM_JOY1BUTTONUP     (WM_MM_RESERVED_FIRST + 7)
#define MM_JOY2BUTTONUP     (1 + MM_JOY1BUTTONUP)

#define MM_MCINOTIFY	    (WM_MM_RESERVED_FIRST + 9)
                            
#define MM_WOM_OPEN         (WM_MM_RESERVED_FIRST + 11)
#define MM_WOM_CLOSE        (WM_MM_RESERVED_FIRST + 12)
#define MM_WOM_DONE         (WM_MM_RESERVED_FIRST + 13)

#define MM_WIM_OPEN         (WM_MM_RESERVED_FIRST + 14)
#define MM_WIM_CLOSE        (WM_MM_RESERVED_FIRST + 15)
#define MM_WIM_DATA         (WM_MM_RESERVED_FIRST + 16)

#define MM_MIM_OPEN         (WM_MM_RESERVED_FIRST + 17)
#define MM_MIM_CLOSE        (WM_MM_RESERVED_FIRST + 18)
#define MM_MIM_DATA         (WM_MM_RESERVED_FIRST + 19)
#define MM_MIM_LONGDATA     (WM_MM_RESERVED_FIRST + 20)
#define MM_MIM_ERROR        (WM_MM_RESERVED_FIRST + 21)
#define MM_MIM_LONGERROR    (WM_MM_RESERVED_FIRST + 22)

#define MM_MOM_OPEN         (WM_MM_RESERVED_FIRST + 23)
#define MM_MOM_CLOSE        (WM_MM_RESERVED_FIRST + 24)
#define MM_MOM_DONE         (WM_MM_RESERVED_FIRST + 25)

#define MM_MCISYSTEM_STRING (WM_MM_RESERVED_FIRST + 26)

// maximum length of szPname in devcaps structures
#define MAXPNAMELEN             32

/****************************************************************************

    String resource number bases

****************************************************************************/

#define MAXERRORLENGTH         128    // maximum error text length (including
				      // terminating null)
// error string resources
#define MMSYSERR_BASE          0
#define WAVERR_BASE            32
#define MIDIERR_BASE           64
#define TIMERR_BASE            128
#define JOYERR_BASE            160
#define MCIERR_BASE            256

#define MCI_STRING_OFFSET      512    // general mci string resources
#define MCI_VD_OFFSET          1024   // videodisc command string resources
#define MCI_CD_OFFSET          1088   // redbook audio command string resources
#define MCI_WAVE_OFFSET        1152   // wave audio command string resources
#define MCI_SEQ_OFFSET         1216   // sequencer command string resources

/****************************************************************************

    General error return values

****************************************************************************/

#define MMSYSERR_NOERROR       0                    // no error
#define MMSYSERR_ERROR         (MMSYSERR_BASE + 1)  // unspecified error
#define MMSYSERR_BADDEVICEID   (MMSYSERR_BASE + 2)  // device id is out of range
#define MMSYSERR_NOTENABLED    (MMSYSERR_BASE + 3)  // driver failed its enable
#define MMSYSERR_ALLOCATED     (MMSYSERR_BASE + 4)  // device already allocated
#define MMSYSERR_INVALHANDLE   (MMSYSERR_BASE + 5)  // device handle is invalid
#define MMSYSERR_NODRIVER      (MMSYSERR_BASE + 6)  // no device driver present
#define MMSYSERR_NOMEM         (MMSYSERR_BASE + 7)  // memory allocation error
#define MMSYSERR_NOTSUPPORTED  (MMSYSERR_BASE + 8)  // function isn't supported
#define MMSYSERR_BADERRNUM     (MMSYSERR_BASE + 9)  // error number out of range
#define MMSYSERR_LASTERROR     (MMSYSERR_BASE + 9)  // last error in range

/****************************************************************************

    driver callback

    the CALLBACK_* flags are used with WaveOutOpen(), WaveInOpen(),
    midiInOpen(), and midiOutOpen() to determine the type of the
    dwCallback parameter

****************************************************************************/

#define CALLBACK_TYPEMASK   0x00070000l    // callback type mask
#define CALLBACK_NULL       0x00000000l    // unknown callback type
#define CALLBACK_WINDOW     0x00010000l    // dwCallback is a HWND
#define CALLBACK_TASK       0x00020000l    // dwCallback is a HTASK
#define CALLBACK_FUNCTION   0x00030000l    // dwCallback is a FARPROC


typedef void (FAR PASCAL DRVCALLBACK) (HANDLE h, WORD msg, DWORD dwUser, DWORD dw1, DWORD dw2);
typedef DRVCALLBACK FAR *LPDRVCALLBACK;

/****************************************************************************

    Manufacturer and product ids

****************************************************************************/

// manufacturer ids
#define MM_MICROSOFT		1

// product ids

#define MM_MIDI_MAPPER          1
#define MM_WAVE_MAPPER          2

#define MM_SNDBLST_MIDIOUT	3
#define MM_SNDBLST_MIDIIN	4
#define MM_SNDBLST_SYNTH 	5
#define MM_SNDBLST_WAVEOUT	6
#define MM_SNDBLST_WAVEIN	7

#define MM_ADLIB		9

#define MM_MPU401_MIDIOUT	10
#define MM_MPU401_MIDIIN	11

#define MM_PC_JOYSTICK		12

/****************************************************************************

    MMTIME union of various structs

****************************************************************************/

typedef struct mmtime_tag {
    WORD    wType;              // the contents of the union
    union {
        DWORD ms;               // milliseconds
        DWORD sample;           // samples
        DWORD cb;               // byte count
        struct {                // SMPTE
            BYTE hour;          // hours
            BYTE min;           // minutes
            BYTE sec;           // seconds
            BYTE frame;         // frames
            BYTE fps;           // frames per second (24, 25, 29(30 drop) or 30)
            BYTE dummy;
            } smpte;
        struct {                // MIDI
	    DWORD songptrpos;   // song pointer position
            } midi;
        } u;
    } MMTIME;
typedef MMTIME FAR *LPMMTIME;

// types for MMTIME struct
#define TIME_MS         0x0001  // time counted in Milliseconds
#define TIME_SAMPLES    0x0002  // number of wave samples
#define TIME_BYTES      0x0004  // current byte offset
#define TIME_SMPTE      0x0008  // smpte time
#define TIME_MIDI       0x0010  // midi time

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

#ifndef MMNOMMSYSTEM

/****************************************************************************

    Version Function for mmsystem - high order byte is major version. low
    order byte is minor version.

****************************************************************************/

WORD FAR PASCAL mmsystemGetVersion(void);

#endif

#ifndef MMNOSOUND
/****************************************************************************

             Sound support functions

****************************************************************************/

BOOL FAR PASCAL sndPlaySound(LPSTR lpszSoundName, WORD wFlags);

// flag values for sndPlaySound()
#define SND_SYNC            0x0000  // play synchronously (default)
#define SND_ASYNC           0x0001  // play asynchronously
#define SND_NODEFAULT       0x0002  // don't use default if sound not found
#define SND_MEMORY          0x0004  // the LPSTR points to a memory file
#define SND_LOOP            0x0008  // loop the sound, until next sndPlaySound
#define SND_NOSTOP          0x0010  // dont stop any currently playing sound

#endif		// ifndef MMNOSOUND

#ifndef MMNOWAVE
/****************************************************************************

		Waveform

****************************************************************************/

typedef HANDLE	HWAVEIN;
typedef HANDLE	HWAVEOUT;
typedef HWAVEIN FAR *LPHWAVEIN;
typedef HWAVEOUT FAR *LPHWAVEOUT;

// device id for wave mapper
#define WAVE_MAPPER     (-1)

// flags for waveOutOpen and waveInOpen
#define  WAVE_FORMAT_QUERY     0x0001

/****************************************************************************

    Wave error return values

****************************************************************************/

#define WAVERR_BADFORMAT      (WAVERR_BASE + 0)  // unsupported wave format
#define WAVERR_STILLPLAYING   (WAVERR_BASE + 1)  // still something playing
#define WAVERR_UNPREPARED     (WAVERR_BASE + 2)  // header hasn't been prepared
#define WAVERR_LASTERROR      (WAVERR_BASE + 2)  // last error in range

/****************************************************************************

    Wave callback

****************************************************************************/

typedef DRVCALLBACK WAVECALLBACK;
typedef WAVECALLBACK FAR *LPWAVECALLBACK;

// wave callback messages
#define WOM_OPEN        MM_WOM_OPEN
#define WOM_CLOSE       MM_WOM_CLOSE
#define WOM_DONE        MM_WOM_DONE

#define WIM_OPEN        MM_WIM_OPEN
#define WIM_CLOSE       MM_WIM_CLOSE
#define WIM_DATA        MM_WIM_DATA

/****************************************************************************
 
    Wave data block header
 
****************************************************************************/

typedef struct wavehdr_tag {
    LPSTR       lpData;                 // pointer to locked data buffer
    DWORD       dwBufferLength;         // length of data buffer
    DWORD       dwBytesRecorded;        // used for input only
    DWORD       dwUser;                 // for client's use
    DWORD       dwFlags;                // assorted flags (see defines)
    DWORD       dwLoops;                // loop control counter
    struct wavehdr_tag far *lpNext;     // reserved for driver
    DWORD       reserved;               // reserved for driver
} WAVEHDR;
typedef WAVEHDR FAR *LPWAVEHDR;

// equates for WAVEHDR flag bits
#define WHDR_DONE       0x00000001  // done bit
#define WHDR_PREPARED   0x00000002  // set if this header has been prepared
#define WHDR_BEGINLOOP  0x00000004  // loop start block
#define WHDR_ENDLOOP    0x00000008  // loop end block 
#define WHDR_INQUEUE    0x00000010  // reserved for driver

/***************************************************************************
 
    Wave device caps structures
 
****************************************************************************/

typedef struct waveoutcaps_tag {
    WORD    wMid;                  // manufacturer id
    WORD    wPid;                  // product id
    VERSION vDriverVersion;        // version of the driver
    char    szPname[MAXPNAMELEN];  // product name (NULL terminated string)
    DWORD   dwFormats;             // formats supported
    WORD    wChannels;	           // number of sources supported.
    DWORD   dwSupport;             // what functionality the driver supports
} WAVEOUTCAPS;
typedef WAVEOUTCAPS FAR *LPWAVEOUTCAPS;

// flags for dwSupport
#define WAVECAPS_PITCH		0x0001   // supports pitch control
#define WAVECAPS_PLAYBACKRATE	0x0002   // supports playback rate control
#define WAVECAPS_VOLUME		0x0004   // supports volume control
#define WAVECAPS_LRVOLUME       0x0008   // separate left-right volume control

typedef struct waveincaps_tag {
    WORD    wMid;                    // manufacturer id
    WORD    wPid;                    // product id
    VERSION vDriverVersion;          // version of the driver
    char    szPname[MAXPNAMELEN];    // product name (NULL terminated string)
    DWORD   dwFormats;		     // formats supported
    WORD    wChannels;		     // number of streams supported.
} WAVEINCAPS;
typedef WAVEINCAPS FAR *LPWAVEINCAPS;

// defines for WAVEXXXCAPS format field
#define WAVE_INVALIDFORMAT     0x00000000	// invalid format
#define WAVE_FORMAT_1M08       0x00000001	// 11.025 kHz, Mono,   8-bit
#define WAVE_FORMAT_1S08       0x00000002	// 11.025 kHz, Stereo, 8-bit
#define WAVE_FORMAT_1M16       0x00000004	// 11.025 kHz, Mono,   16-bit
#define WAVE_FORMAT_1S16       0x00000008	// 11.025 kHz, Stereo, 16-bit
#define WAVE_FORMAT_2M08       0x00000010	// 22.05  kHz, Mono,   8-bit
#define WAVE_FORMAT_2S08       0x00000020	// 22.05  kHz, Stereo, 8-bit
#define WAVE_FORMAT_2M16       0x00000040	// 22.05  kHz, Mono,   16-bit
#define WAVE_FORMAT_2S16       0x00000080	// 22.05  kHz, Stereo, 16-bit
#define WAVE_FORMAT_4M08       0x00000100	// 44.1   kHz, Mono,   8-bit
#define WAVE_FORMAT_4S08       0x00000200	// 44.1   kHz, Stereo, 8-bit
#define WAVE_FORMAT_4M16       0x00000400	// 44.1   kHz, Mono,   16-bit
#define WAVE_FORMAT_4S16       0x00000800	// 44.1   kHz, Stereo, 16-bit

/**************************************************************************** 
 
   Wave format structures
 
****************************************************************************/

// format information common to all formats
typedef struct waveformat_tag {
    WORD    wFormatTag;        // what format type is this?
    WORD    nChannels;         // number of channels (i.e. mono, stereo, etc.)
    DWORD   nSamplesPerSec;    // sample rate
    DWORD   nAvgBytesPerSec;   // for buffer estimation
    WORD    nBlockAlign;       // block size of data
} WAVEFORMAT;
typedef WAVEFORMAT NEAR *NPWAVEFORMAT;
typedef WAVEFORMAT FAR *LPWAVEFORMAT;
     
// defines for wFormatTag
#define WAVE_FORMAT_PCM     1
     
// use if wFormatTag == WAVE_FORMAT_PCM
typedef struct pcmwaveformat_tag {
    WAVEFORMAT  wf;
    WORD        wBitsPerSample;
} PCMWAVEFORMAT;
typedef PCMWAVEFORMAT NEAR *NPPCMWAVEFORMAT;
typedef PCMWAVEFORMAT FAR *LPPCMWAVEFORMAT;

/**************************************************************************** 
 
   Wave support functions
 
****************************************************************************/

WORD FAR PASCAL waveOutGetNumDevs(void);
WORD FAR PASCAL waveOutGetDevCaps(WORD wDeviceID,
					LPWAVEOUTCAPS lpCaps, WORD wSize);
WORD FAR PASCAL waveOutGetVolume(WORD wDeviceID, LPDWORD lpdwVolume);
WORD FAR PASCAL waveOutSetVolume(WORD wDeviceID, DWORD dwVolume);
WORD FAR PASCAL waveOutGetErrorText(WORD wError, LPSTR lpText, WORD wSize);
WORD FAR PASCAL waveOutOpen(LPHWAVEOUT lphWaveOut, WORD wDeviceID, 
                            LPWAVEFORMAT lpFormat, DWORD dwCallback,
                            DWORD dwInstance, DWORD dwFlags);
WORD FAR PASCAL waveOutClose(HWAVEOUT hWaveOut);
WORD FAR PASCAL waveOutPrepareHeader(HWAVEOUT hWaveOut,
					LPWAVEHDR lpWaveOutHdr, WORD wSize);
WORD FAR PASCAL waveOutUnprepareHeader(HWAVEOUT hWaveOut,
					LPWAVEHDR lpWaveOutHdr, WORD wSize);
WORD FAR PASCAL waveOutWrite(HWAVEOUT hWaveOut,
					LPWAVEHDR lpWaveOutHdr, WORD wSize);
WORD FAR PASCAL waveOutPause(HWAVEOUT hWaveOut);
WORD FAR PASCAL waveOutRestart(HWAVEOUT hWaveOut);
WORD FAR PASCAL waveOutReset(HWAVEOUT hWaveOut);
WORD FAR PASCAL waveOutBreakLoop(HWAVEOUT hWaveOut);
WORD FAR PASCAL waveOutGetPosition(HWAVEOUT hWaveOut,
					LPMMTIME lpInfo, WORD wSize);
WORD FAR PASCAL waveOutGetPitch(HWAVEOUT hWaveOut, LPDWORD lpdwPitch);
WORD FAR PASCAL waveOutSetPitch(HWAVEOUT hWaveOut, DWORD dwPitch);
WORD FAR PASCAL waveOutGetPlaybackRate(HWAVEOUT hWaveOut, LPDWORD lpdwRate);
WORD FAR PASCAL waveOutSetPlaybackRate(HWAVEOUT hWaveOut, DWORD dwRate);
WORD FAR PASCAL waveOutGetID(HWAVEOUT hWaveOut, LPWORD lpwDeviceID);

WORD FAR PASCAL waveInGetNumDevs(void);
WORD FAR PASCAL waveInGetDevCaps(WORD wDeviceID,
					LPWAVEINCAPS lpCaps, WORD wSize);
WORD FAR PASCAL waveInGetErrorText(WORD wError, LPSTR lpText, WORD wSize);
WORD FAR PASCAL waveInOpen(LPHWAVEIN lphWaveIn, WORD wDeviceID,
                           LPWAVEFORMAT lpFormat, DWORD dwCallback,
                           DWORD dwInstance, DWORD dwFlags);
WORD FAR PASCAL waveInClose(HWAVEIN hWaveIn);
WORD FAR PASCAL waveInPrepareHeader(HWAVEIN hWaveIn,
					LPWAVEHDR lpWaveInHdr, WORD wSize);
WORD FAR PASCAL waveInUnprepareHeader(HWAVEIN hWaveIn,
					LPWAVEHDR lpWaveInHdr, WORD wSize);
WORD FAR PASCAL waveInAddBuffer(HWAVEIN hWaveIn,
					LPWAVEHDR lpWaveInHdr, WORD wSize);
WORD FAR PASCAL waveInStart(HWAVEIN hWaveIn);
WORD FAR PASCAL waveInStop(HWAVEIN hWaveIn);
WORD FAR PASCAL waveInReset(HWAVEIN hWaveIn);
WORD FAR PASCAL waveInGetPosition(HWAVEIN hWaveIn, LPMMTIME lpInfo, WORD wSize);
WORD FAR PASCAL waveInGetID(HWAVEIN hWaveIn, LPWORD lpwDeviceID);

#endif		// ifndef MMNOWAVE

#ifndef MMNOMIDI
/****************************************************************************

		MIDI

****************************************************************************/

typedef HANDLE	HMIDIIN;
typedef HANDLE	HMIDIOUT;
typedef HMIDIIN FAR *LPHMIDIIN;
typedef HMIDIOUT FAR *LPHMIDIOUT;

// device id for mapper
#define MIDIMAPPER     (-1)

/****************************************************************************

    MIDI error return values

****************************************************************************/

#define MIDIERR_UNPREPARED    (MIDIERR_BASE + 0)  // header hasn't been prepared
#define MIDIERR_STILLPLAYING  (MIDIERR_BASE + 1)  // still something playing
#define MIDIERR_NOMAP         (MIDIERR_BASE + 2)  // no current map
#define MIDIERR_NOTREADY      (MIDIERR_BASE + 3)  // hardware is still busy
#define MIDIERR_NODEVICE      (MIDIERR_BASE + 4)  // port no longer connected
#define MIDIERR_LASTERROR     (MIDIERR_BASE + 4)  // last error in range

/****************************************************************************

    MIDI callback

****************************************************************************/

typedef DRVCALLBACK MIDICALLBACK;
typedef MIDICALLBACK FAR *LPMIDICALLBACK;

// callback messages
#define MIM_OPEN        MM_MIM_OPEN
#define MIM_CLOSE       MM_MIM_CLOSE
#define MIM_DATA        MM_MIM_DATA
#define MIM_LONGDATA    MM_MIM_LONGDATA
#define MIM_ERROR       MM_MIM_ERROR
#define MIM_LONGERROR   MM_MIM_LONGERROR

#define MOM_OPEN        MM_MOM_OPEN
#define MOM_CLOSE       MM_MOM_CLOSE
#define MOM_DONE        MM_MOM_DONE

/****************************************************************************
 
    MIDI device caps structures
 
*****************************************************************************/

// MIDI output device caps
typedef struct midioutcaps_tag { 
    WORD    wMid;                  // manufacturer id
    WORD    wPid;                  // product id
    VERSION vDriverVersion;        // version of the driver
    char    szPname[MAXPNAMELEN];  // product name (NULL terminated string)
    WORD    wTechnology;           // type of device
    WORD    wVoices;               // number of voices          
    WORD    wNotes;                // max number of notes
    WORD    wChannelMask;          // channels on port which device is attached
    DWORD   dwSupport;             // what functionality the driver supports
} MIDIOUTCAPS;
typedef MIDIOUTCAPS FAR *LPMIDIOUTCAPS;
 
// defines for MIDIOUTCAPS technology field
#define MOD_MIDIPORT    1  // MIDI hardware port
#define MOD_SYNTH       2  // MIDI synthesizer (unspecified type)
#define MOD_SQSYNTH     3  // square wave synth
#define MOD_FMSYNTH     4  // FM synth
#define MOD_MAPPER      5  // MIDI mapper

// flags for dwSupport
#define MIDICAPS_VOLUME          0x0001	 // supports volume control
#define MIDICAPS_LRVOLUME        0x0002  // separate left-right volume control
#define MIDICAPS_CACHE           0x0004  // supports patch cacheing

// MIDI input device caps
typedef struct midiincaps_tag { 
    WORD    wMid;                  // manufacturer id
    WORD    wPid;                  // product id
    VERSION vDriverVersion;        // version of the driver
    char    szPname[MAXPNAMELEN];  // product name (NULL terminated string)
} MIDIINCAPS;
typedef MIDIINCAPS FAR *LPMIDIINCAPS;

/****************************************************************************
 
    MIDI data block header
 
*****************************************************************************/

typedef struct midihdr_tag {
    LPSTR       lpData;               // pointer to locked data block
    DWORD       dwBufferLength;       // length of data in data block
    DWORD       dwBytesRecorded;      // used for input only
    DWORD       dwUser;               // for client's use
    DWORD       dwFlags;              // assorted flags (see defines)
    struct midihdr_tag far *lpNext;   // reserved for driver
    DWORD       reserved;             // reserved for driver
} MIDIHDR;
typedef MIDIHDR FAR *LPMIDIHDR;

// defines for MIDIHDR flag bits
#define MHDR_DONE       0x00000001       // done bit
#define MHDR_PREPARED   0x00000002       // set if this header has been prepared
#define MHDR_INQUEUE    0x00000004       // reserved for driver

/****************************************************************************
 
    MIDI patch cache arrays
 
*****************************************************************************/

typedef WORD PATCHARRAY[128];
typedef WORD FAR *LPPATCHARRAY;

typedef WORD KEYARRAY[128];
typedef WORD FAR *LPKEYARRAY;

// flags for midiOutCachePatches
#define MIDI_CACHE_ALL      1
#define MIDI_CACHE_BESTFIT  2
#define MIDI_CACHE_QUERY    3
#define MIDI_UNCACHE        4

/**************************************************************************** 

    MIDI support functions

****************************************************************************/

WORD FAR PASCAL midiOutGetNumDevs(void);
WORD FAR PASCAL midiOutGetDevCaps(WORD wDeviceID,
					LPMIDIOUTCAPS lpCaps, WORD wSize);
WORD FAR PASCAL midiOutGetVolume(WORD wDeviceID, LPDWORD lpdwVolume);
WORD FAR PASCAL midiOutSetVolume(WORD wDeviceID, DWORD dwVolume);
WORD FAR PASCAL midiOutGetErrorText(WORD wError, LPSTR lpText, WORD wSize);
WORD FAR PASCAL midiOutOpen(LPHMIDIOUT lphMidiOut, WORD wDeviceID,
        DWORD dwCallback, DWORD dwInstance, DWORD dwFlags);
WORD FAR PASCAL midiOutClose(HMIDIOUT hMidiOut);
WORD FAR PASCAL midiOutPrepareHeader(HMIDIOUT hMidiOut,
					LPMIDIHDR lpMidiOutHdr, WORD wSize);
WORD FAR PASCAL midiOutUnprepareHeader(HMIDIOUT hMidiOut,
					LPMIDIHDR lpMidiOutHdr, WORD wSize);
WORD FAR PASCAL midiOutShortMsg(HMIDIOUT hMidiOut, DWORD dwMsg);
WORD FAR PASCAL midiOutLongMsg(HMIDIOUT hMidiOut,
					LPMIDIHDR lpMidiOutHdr, WORD wSize);
WORD FAR PASCAL midiOutReset(HMIDIOUT hMidiOut);
WORD FAR PASCAL midiOutCachePatches(HMIDIOUT hMidiOut, WORD wBank,
                                  LPPATCHARRAY lpPatchArray, WORD wFlags);
WORD FAR PASCAL midiOutCacheDrumPatches(HMIDIOUT hMidiOut, WORD wPatch,
                                  LPKEYARRAY lpKeyArray, WORD wFlags);
WORD FAR PASCAL midiOutGetID(HMIDIOUT hMidiOut, LPWORD lpwDeviceID);

WORD FAR PASCAL midiInGetNumDevs(void);
WORD FAR PASCAL midiInGetDevCaps(WORD wDeviceID,
					LPMIDIINCAPS lpCaps, WORD wSize);
WORD FAR PASCAL midiInGetErrorText(WORD wError, LPSTR lpText, WORD wSize);
WORD FAR PASCAL midiInOpen(LPHMIDIIN lphMidiIn, WORD wDeviceID,
        DWORD dwCallback, DWORD dwInstance, DWORD dwFlags);
WORD FAR PASCAL midiInClose(HMIDIIN hMidiIn);
WORD FAR PASCAL midiInPrepareHeader(HMIDIIN hMidiIn,
					LPMIDIHDR lpMidiInHdr, WORD wSize);
WORD FAR PASCAL midiInUnprepareHeader(HMIDIIN hMidiIn,
					LPMIDIHDR lpMidiInHdr, WORD wSize);
WORD FAR PASCAL midiInAddBuffer(HMIDIIN hMidiIn,
					LPMIDIHDR lpMidiInHdr, WORD wSize);
WORD FAR PASCAL midiInStart(HMIDIIN hMidiIn);
WORD FAR PASCAL midiInStop(HMIDIIN hMidiIn);
WORD FAR PASCAL midiInReset(HMIDIIN hMidiIn);
WORD FAR PASCAL midiInGetID(HMIDIIN hMidiIn, LPWORD lpwDeviceID);

#endif		// ifndef MMNOMIDI

#ifndef MMNOAUX
/****************************************************************************

    auxiliary output device

****************************************************************************/

/***************************************************************************
 
    aux device caps structures
 
****************************************************************************/

typedef struct auxcaps_tag {
    WORD    wMid;                  // manufacturer id
    WORD    wPid;                  // product id
    VERSION vDriverVersion;        // version of the driver
    char    szPname[MAXPNAMELEN];  // product name (NULL terminated string)
    WORD    wTechnology;           // type of device
    DWORD   dwSupport;             // what functionality the driver supports
} AUXCAPS;
typedef AUXCAPS FAR *LPAUXCAPS;

// defines for AUXCAPS technology field
#define AUXCAPS_CDAUDIO    1       // audio from internal CD-ROM drive
#define AUXCAPS_AUXIN      2       // audio from auxiliary input jacks

// flags for dwSupport
#define AUXCAPS_VOLUME          0x0001	// supports volume control
#define AUXCAPS_LRVOLUME        0x0002  // separate left-right volume control

/**************************************************************************** 

    aux support functions

****************************************************************************/

WORD FAR PASCAL auxGetNumDevs(void);
WORD FAR PASCAL auxGetDevCaps(WORD wDeviceID, LPAUXCAPS lpCaps, WORD wSize);
WORD FAR PASCAL auxSetVolume(WORD wDeviceID, DWORD dwVolume);
WORD FAR PASCAL auxGetVolume(WORD wDeviceID, LPDWORD lpdwVolume);
#endif		// ifndef MMNOAUX

#ifndef MMNOTIMER
/****************************************************************************

		Timer

****************************************************************************/


/****************************************************************************

    Timer error return values

****************************************************************************/

#define TIMERR_NOERROR   (0)              // no error
#define TIMERR_NOCANDO	 (TIMERR_BASE+1)  // requested operation wasn't executed

/****************************************************************************

    Timer callback

****************************************************************************/

typedef DRVCALLBACK TIMECALLBACK;
typedef TIMECALLBACK FAR *LPTIMECALLBACK;

/****************************************************************************

    Timer device caps structure

****************************************************************************/

typedef struct timecaps_tag {
    WORD    wPeriodMin;	    // minimum period supported.
    WORD    wPeriodMax;     // maximum period supported.
    } TIMECAPS;
typedef TIMECAPS FAR *LPTIMECAPS;

/****************************************************************************

    Timer support functions

****************************************************************************/

WORD FAR PASCAL timeGetSystemTime(LPMMTIME lpTime, WORD wSize);
DWORD FAR PASCAL timeGetTime(void);
WORD FAR PASCAL timeSetEvent(WORD wDelay, WORD wResolution, 
    LPTIMECALLBACK lpFunction, DWORD dwUser, WORD wFlags); 
WORD FAR PASCAL timeKillEvent(WORD wId);
WORD FAR PASCAL timeGetDevCaps(LPTIMECAPS lpTimeCaps, WORD wSize);
void FAR PASCAL timeBeginPeriod(WORD wPeriod);
void FAR PASCAL timeEndPeriod(WORD wPeriod);


// flags for timeSetEvent
#define TIME_ONESHOT	0   // program timer for single event
#define TIME_PERIODIC	1   // program for continuous periodic event

#endif		// ifndef MMNOTIMER

#ifndef MMNOJOY
/****************************************************************************

		Joystick

****************************************************************************/

/****************************************************************************

    Joystick error return values

****************************************************************************/

#define JOYERR_NOERROR	  (0)               // no error
#define JOYERR_PARMS	  (JOYERR_BASE+5)   // bad parameters passed to function
#define JOYERR_NOCANDO	  (JOYERR_BASE+6)   // cannot execute required function 
			    // due to some service being unavailable (ex timer)
#define JOYERR_UNPLUGGED  (JOYERR_BASE+7)   // joystick specified is unplugged

/****************************************************************************

    Joystick device caps structure

****************************************************************************/

typedef struct joycaps_tag {
    WORD wMid;                  // manufacturer id
    WORD wPid;                  // product id
    char szPname[MAXPNAMELEN];  // product name (NULL terminated string)
    WORD wXmin;                 // minimum x position value
    WORD wXmax;                 // maximum x position value
    WORD wYmin;                 // minimum y position value
    WORD wYmax;                 // maximum y position value
    WORD wZmin; 		// minimum z position value
    WORD wZmax; 		// maximum z position value
    WORD wNumButtons;		// number of buttons
    WORD wPeriodMin;		// minimum message period when captured
    WORD wPeriodMax;		// maximum message period when captured
    } JOYCAPS;

typedef JOYCAPS FAR *LPJOYCAPS;

/****************************************************************************

    Joystick information structure

****************************************************************************/

typedef struct joyinfo_tag {
    WORD wXpos;                 // x position
    WORD wYpos;                 // y position
    WORD wZpos; 		// z position
    WORD wButtons;              // button states
    } JOYINFO;

typedef JOYINFO FAR *LPJOYINFO;

/****************************************************************************

   Joystick support functions

****************************************************************************/

WORD FAR PASCAL joyGetDevCaps(WORD id, LPJOYCAPS lpCaps, WORD wSize);
WORD FAR PASCAL joyGetNumDevs(void);
WORD FAR PASCAL joyGetPos(WORD id, LPJOYINFO lpInfo);
WORD FAR PASCAL joyGetThreshold(WORD id,LPWORD lpwThreshold);
WORD FAR PASCAL joyReleaseCapture(WORD id);
WORD FAR PASCAL joySetCapture(HWND hwnd, WORD wId, WORD wPeriod, BOOL bChanged);
WORD FAR PASCAL joySetThreshold(WORD id, WORD wThreshold);

WORD FAR PASCAL joySetCalibration(WORD id, LPWORD pwXbase, LPWORD pwXdelta,
				  LPWORD pwYbase, LPWORD pwYdelta,
				  LPWORD pwZbase, LPWORD pwZdelta); // internal

/****************************************************************************

    Joystick messages

****************************************************************************/

#define JOY_BUTTON1         0x0001
#define JOY_BUTTON2	    0x0002
#define JOY_BUTTON3	    0x0004
#define JOY_BUTTON4	    0x0008
#define JOY_BUTTON1CHG      0x0100
#define JOY_BUTTON2CHG      0x0200
#define JOY_BUTTON3CHG	    0x0400
#define JOY_BUTTON4CHG	    0x0800

#define JOYSTICKID1	    0
#define JOYSTICKID2	    1

#endif		// ifndef MMNOJOY

#ifndef MMNODRV // installable driver support

#define DRV_CANCEL             0x0000 
#define DRV_OK                 0x0001   
#define DRV_RESTART            0x0002

#define DRV_LOAD               0x0001
#define DRV_ENABLE             0x0002
#define DRV_OPEN               0x0003
#define DRV_CLOSE              0x0004
#define DRV_DISABLE            0x0005
#define DRV_FREE               0x0006
#define DRV_CONFIGURE          0x0007
#define DRV_QUERYCONFIGURE     0x0008
#define DRV_INSTALL            0x0009
#define DRV_REMOVE             0x000A

#define DRV_RESERVED           0x0800

#define DRV_MCI_FIRST          DRV_RESERVED
#define DRV_MCI_LAST           (DRV_RESERVED + 0xFFF)

#define DRV_USER               0x4000

typedef struct tagDRVCONFIGINFO
    {
    DWORD   dwDCISize;
    LPSTR   lpszDCISectionName;
    LPSTR   lpszDCIAliasName;
    } DRVCONFIGINFO;
typedef DRVCONFIGINFO       *PDRVCONFIGINFO;
typedef DRVCONFIGINFO FAR   *LPDRVCONFIGINFO;

LONG   FAR PASCAL DrvClose(HANDLE hDriver, LONG lParam1, LONG lParam2);
HANDLE FAR PASCAL DrvOpen(LPSTR szDriverName, LPSTR szSectionName, LONG lParam2);
LONG   FAR PASCAL DrvSendMessage(HANDLE hDriver, WORD message, LONG lParam1, LONG lParam2);
HANDLE FAR PASCAL DrvGetModuleHandle(HANDLE hDriver);
LONG   FAR PASCAL DefDriverProc(DWORD dwDriverIdentifier, HANDLE driverID, WORD message, LONG lParam1, LONG lParam2);

#endif

#ifndef MMNOMCI
/****************************************************************************

		MCI

****************************************************************************/


/****************************************************************************

    MCI error return values

*****************************************************************************/

#define MCIERR_INVALID_DEVICE_ID        (MCIERR_BASE + 1)
#define MCIERR_UNRECOGNIZED_KEYWORD     (MCIERR_BASE + 3)
#define MCIERR_UNRECOGNIZED_COMMAND     (MCIERR_BASE + 5)
#define MCIERR_HARDWARE                 (MCIERR_BASE + 6)
#define MCIERR_INVALID_DEVICE_NAME      (MCIERR_BASE + 7)
#define MCIERR_OUT_OF_MEMORY            (MCIERR_BASE + 8)
#define MCIERR_DEVICE_OPEN              (MCIERR_BASE + 9)
#define MCIERR_CANNOT_LOAD_DRIVER       (MCIERR_BASE + 10)
#define MCIERR_MISSING_COMMAND_STRING   (MCIERR_BASE + 11)
#define MCIERR_PARAM_OVERFLOW           (MCIERR_BASE + 12)
#define MCIERR_MISSING_STRING_ARGUMENT  (MCIERR_BASE + 13)
#define MCIERR_BAD_INTEGER              (MCIERR_BASE + 14)
#define MCIERR_PARSER_INTERNAL          (MCIERR_BASE + 15)
#define MCIERR_DRIVER_INTERNAL          (MCIERR_BASE + 16)
#define MCIERR_MISSING_PARAMETER        (MCIERR_BASE + 17)
#define MCIERR_UNSUPPORTED_FUNCTION     (MCIERR_BASE + 18)
#define MCIERR_FILE_NOT_FOUND           (MCIERR_BASE + 19)
#define MCIERR_DEVICE_NOT_READY         (MCIERR_BASE + 20)
#define MCIERR_INTERNAL                 (MCIERR_BASE + 21)
#define MCIERR_DRIVER                   (MCIERR_BASE + 22)
#define MCIERR_CANNOT_USE_ALL           (MCIERR_BASE + 23)
#define MCIERR_MULTIPLE                 (MCIERR_BASE + 24)
#define MCIERR_EXTENSION_NOT_FOUND      (MCIERR_BASE + 25)
#define MCIERR_OUTOFRANGE	        (MCIERR_BASE + 26)
#define MCIERR_FLAGS_NOT_COMPATIBLE     (MCIERR_BASE + 28)
#define MCIERR_FILE_NOT_SAVED           (MCIERR_BASE + 30)
#define MCIERR_DEVICE_TYPE_REQUIRED     (MCIERR_BASE + 31)
#define MCIERR_DEVICE_LOCKED            (MCIERR_BASE + 32)
#define MCIERR_DUPLICATE_ALIAS          (MCIERR_BASE + 33)
#define MCIERR_BAD_CONSTANT             (MCIERR_BASE + 34)
#define MCIERR_MUST_USE_SHAREABLE       (MCIERR_BASE + 35)
#define MCIERR_MISSING_DEVICE_NAME      (MCIERR_BASE + 36)
#define MCIERR_BAD_TIME_FORMAT          (MCIERR_BASE + 37)
#define MCIERR_NO_CLOSING_QUOTE         (MCIERR_BASE + 38)
#define MCIERR_DUPLICATE_FLAGS          (MCIERR_BASE + 39)
#define	MCIERR_INVALID_FILE		(MCIERR_BASE + 40)
#define MCIERR_NULL_PARAMETER_BLOCK     (MCIERR_BASE + 41)
#define	MCIERR_UNNAMED_RESOURCE		(MCIERR_BASE + 42)
#define MCIERR_NEW_REQUIRES_ALIAS       (MCIERR_BASE + 43)
#define MCIERR_NOTIFY_ON_AUTO_OPEN      (MCIERR_BASE + 44)
#define MCIERR_NO_ELEMENT_ALLOWED       (MCIERR_BASE + 45)
#define MCIERR_NONAPPLICABLE_FUNCTION   (MCIERR_BASE + 46)
#define MCIERR_ILLEGAL_FOR_AUTO_OPEN    (MCIERR_BASE + 47)
#define MCIERR_FILENAME_REQUIRED        (MCIERR_BASE + 48)
#define MCIERR_EXTRA_CHARACTERS         (MCIERR_BASE + 49)

#define MCIERR_WAVE_OUTPUTSINUSE        (MCIERR_BASE + 64)
#define MCIERR_WAVE_SETOUTPUTINUSE      (MCIERR_BASE + 65)
#define MCIERR_WAVE_INPUTSINUSE         (MCIERR_BASE + 66)
#define MCIERR_WAVE_SETINPUTINUSE       (MCIERR_BASE + 67)
#define MCIERR_WAVE_OUTPUTUNSPECIFIED   (MCIERR_BASE + 68)
#define MCIERR_WAVE_INPUTUNSPECIFIED    (MCIERR_BASE + 69)
#define MCIERR_WAVE_OUTPUTSUNSUITABLE   (MCIERR_BASE + 70)
#define MCIERR_WAVE_SETOUTPUTUNSUITABLE (MCIERR_BASE + 71)
#define MCIERR_WAVE_INPUTSUNSUITABLE    (MCIERR_BASE + 72)
#define MCIERR_WAVE_SETINPUTUNSUITABLE  (MCIERR_BASE + 73)


#define MCIERR_SEQ_DIV_INCOMPATIBLE	(MCIERR_BASE + 80)
#define MCIERR_SEQ_PORT_INUSE		(MCIERR_BASE + 81)
#define MCIERR_SEQ_PORT_NONEXISTENT	(MCIERR_BASE + 82)
#define MCIERR_SEQ_PORT_MAPNODEVICE	(MCIERR_BASE + 83)
#define MCIERR_SEQ_PORT_MISCERROR	(MCIERR_BASE + 84)
#define MCIERR_SEQ_TIMER		(MCIERR_BASE + 85)
#define MCIERR_SEQ_PORTUNSPECIFIED      (MCIERR_BASE + 86)

#define MCIERR_NO_WINDOW                (MCIERR_BASE + 90)
#define MCIERR_CREATEWINDOW             (MCIERR_BASE + 91)
#define MCIERR_FILE_READ                (MCIERR_BASE + 92) 
#define MCIERR_FILE_WRITE               (MCIERR_BASE + 93)

// All custom device driver errors must be greater than this value
#define MCIERR_CUSTOM_DRIVER_BASE       (MCIERR_BASE + 256)

/****************************************************************************

    MCI command message identifiers

*****************************************************************************/
// Message numbers are hard coded because the resource compiler
// will otherwise fail on the RCDATA command table in mmsystem.rc

// They must be in the range between DRV_MCI_FIRST and DRV_MCI_LAST

// Messages 0x801 and 0x802 are reserved
#define MCI_OPEN                        0x0803
#define MCI_CLOSE                       0x0804
#define MCI_ESCAPE                      0x0805
#define MCI_PLAY                        0x0806
#define MCI_SEEK                        0x0807
#define MCI_STOP                        0x0808
#define MCI_PAUSE                       0x0809
#define MCI_INFO                        0x080A
#define MCI_GETDEVCAPS                  0x080B
#define MCI_SPIN                        0x080C
#define MCI_SET                         0x080D
#define MCI_STEP                        0x080E
#define MCI_RECORD                      0x080F
#define MCI_SYSINFO                     0x0810
#define MCI_BREAK                       0x0811
#define MCI_SOUND                       0x0812
#define MCI_SAVE                        0x0813
#define MCI_STATUS                      0x0814

#define MCI_CUE                         0x0830

#define MCI_REALIZE                     0x0840
#define MCI_WINDOW                      0x0841
#define MCI_PUT                         0x0842
#define MCI_WHERE                       0x0843
#define MCI_FREEZE                      0x0844
#define MCI_UNFREEZE                    0x0845

#define MCI_LOAD                        0x0850
#define MCI_CUT                         0x0851
#define MCI_COPY                        0x0852
#define MCI_PASTE                       0x0853
#define MCI_UPDATE                      0x0854
#define MCI_RESUME                      0x0855
#define MCI_DELETE                      0x0856

// the next 0x400 message ID's are reserved for custom drivers
#define MCI_USER_MESSAGES               (0x400 + DRV_MCI_FIRST)

/****************************************************************************

    Structures for the lpdwParams (dwParam2) of mciSendCommand for those
    command messages that may be parsed in string form.

*****************************************************************************/

// string resource ID's for the MCI core

// Device ID for "all"
#define MCI_ALL_DEVICE_ID               0xFFFF

// Predefined MCI device types
#define MCI_DEVTYPE_VCR                 (MCI_STRING_OFFSET + 1)
#define MCI_DEVTYPE_VIDEODISC           (MCI_STRING_OFFSET + 2)
#define MCI_DEVTYPE_OVERLAY             (MCI_STRING_OFFSET + 3)
#define MCI_DEVTYPE_CD_AUDIO            (MCI_STRING_OFFSET + 4)
#define MCI_DEVTYPE_DAT                 (MCI_STRING_OFFSET + 5)
#define MCI_DEVTYPE_SCANNER             (MCI_STRING_OFFSET + 6)
#define MCI_DEVTYPE_ANIMATION           (MCI_STRING_OFFSET + 7)
#define MCI_DEVTYPE_DIGITAL_VIDEO       (MCI_STRING_OFFSET + 8)
#define MCI_DEVTYPE_OTHER               (MCI_STRING_OFFSET + 9)
#define MCI_DEVTYPE_WAVEFORM_AUDIO      (MCI_STRING_OFFSET + 10)
#define MCI_DEVTYPE_SEQUENCER           (MCI_STRING_OFFSET + 11)

#define MCI_DEVTYPE_FIRST               MCI_DEVTYPE_VCR
#define MCI_DEVTYPE_LAST                MCI_DEVTYPE_SEQUENCER

// Return values for 'status mode'
#define MCI_MODE_NOT_READY              (MCI_STRING_OFFSET + 12)
#define MCI_MODE_STOP                   (MCI_STRING_OFFSET + 13)
#define MCI_MODE_PLAY                   (MCI_STRING_OFFSET + 14)
#define MCI_MODE_RECORD                 (MCI_STRING_OFFSET + 15)
#define MCI_MODE_SEEK                   (MCI_STRING_OFFSET + 16)
#define MCI_MODE_PAUSE                  (MCI_STRING_OFFSET + 17)
#define MCI_MODE_OPEN                   (MCI_STRING_OFFSET + 18)

// Note:  MCI_STRING_OFFSET+19 thru MCI_STRING_OFFSET+31 are reserved

// Core values used in 'set time format' and 'status time format'
#define MCI_FORMAT_MILLISECONDS         0
#define MCI_FORMAT_HMS                  1
#define MCI_FORMAT_MSF                  2
#define MCI_FORMAT_FRAMES               3
#define MCI_FORMAT_SMPTE_24		4
#define MCI_FORMAT_SMPTE_25		5
#define MCI_FORMAT_SMPTE_30		6
#define MCI_FORMAT_SMPTE_30DROP	        7
#define MCI_FORMAT_BYTES                8
#define MCI_FORMAT_SAMPLES              9
#define MCI_FORMAT_TMSF                 10

//
// Time format conversion macros
//
#define MCI_MSF_MINUTE(msf)             ((BYTE)(msf))
#define MCI_MSF_SECOND(msf)             ((BYTE)(((WORD)(msf)) >> 8))
#define MCI_MSF_FRAME(msf)              ((BYTE)((msf)>>16))

#define MCI_MAKE_MSF(m, s, f)           ((DWORD)(((BYTE)(m) | \
                                                  ((WORD)(s)<<8)) | \
                                                 (((DWORD)(BYTE)(f))<<16)))
                                        
#define MCI_TMSF_TRACK(tmsf)            ((BYTE)(tmsf))
#define MCI_TMSF_MINUTE(tmsf)           ((BYTE)(((WORD)(tmsf)) >> 8))
#define MCI_TMSF_SECOND(tmsf)           ((BYTE)((tmsf)>>16))
#define MCI_TMSF_FRAME(tmsf)            ((BYTE)((tmsf)>>24))

#define MCI_MAKE_TMSF(t, m, s, f)       ((DWORD)(((BYTE)(t) | \
                                                  ((WORD)(m)<<8)) | \
                                                 (((DWORD) \
                                                   (BYTE)(s) | \
                                                   ((WORD)(f)<<8)) \
                                                   <<16)))

#define MCI_HMS_HOUR(hms)               ((BYTE)(hms))
#define MCI_HMS_MINUTE(hms)             ((BYTE)(((WORD)(hms)) >> 8))
#define MCI_HMS_SECOND(hms)             ((BYTE)((hms)>>16))

#define MCI_MAKE_HMS(h, m, s)           ((DWORD)(((BYTE)(h) | \
                                                  ((WORD)(m)<<8)) | \
                                                 (((DWORD)(BYTE)(s))<<16)))
                                        

// Flags for mciDriverNotify
#define MCI_NOTIFY_SUCCESSFUL           0x0001
#define MCI_NOTIFY_SUPERSEDED           0x0002
#define MCI_NOTIFY_ABORTED              0x0004
#define MCI_NOTIFY_FAILURE              0x0008

// NOTE: All flags used in dwParam1 of mciSendCommand() must have an
// "L" suffix in order to be parsed as DWORDs by the resource compiler

// Common core flags
#define MCI_NOTIFY                      0x00000001L
#define MCI_WAIT                        0x00000002L
#define MCI_FROM                        0x00000004L
#define MCI_TO                          0x00000008L
#define MCI_TRACK                       0x00000010L

#define MCI_OPEN_SHAREABLE              0x00000100L
#define MCI_OPEN_ELEMENT                0x00000200L
#define MCI_OPEN_ALIAS                  0x00000400L
#define MCI_OPEN_ELEMENT_ID             0x00000800L
#define MCI_OPEN_TYPE_ID                0x00001000L
#define MCI_OPEN_TYPE                   0x00002000L

#define MCI_SEEK_TO_START               0x00000100L
#define MCI_SEEK_TO_END                 0x00000200L

#define MCI_STATUS_ITEM                 0x00000100L
#define MCI_STATUS_START                0x00000200L

#define MCI_STATUS_LENGTH               0x00000001L
#define MCI_STATUS_POSITION             0x00000002L
#define MCI_STATUS_NUMBER_OF_TRACKS     0x00000003L
#define MCI_STATUS_MODE                 0x00000004L
#define MCI_STATUS_MEDIA_PRESENT        0x00000005L
#define MCI_STATUS_TIME_FORMAT          0x00000006L
#define MCI_STATUS_READY                0x00000007L
#define MCI_STATUS_CURRENT_TRACK        0x00000008L

#define MCI_INFO_PRODUCT                0x00000100L
#define MCI_INFO_FILE                   0x00000200L

#define MCI_GETDEVCAPS_ITEM             0x00000100L

#define MCI_GETDEVCAPS_CAN_RECORD       0x00000001L
#define MCI_GETDEVCAPS_HAS_AUDIO        0x00000002L
#define MCI_GETDEVCAPS_HAS_VIDEO        0x00000003L
#define MCI_GETDEVCAPS_DEVICE_TYPE      0x00000004L
#define MCI_GETDEVCAPS_USES_FILES       0x00000005L
#define MCI_GETDEVCAPS_COMPOUND_DEVICE  0x00000006L
#define MCI_GETDEVCAPS_CAN_EJECT        0x00000007L
#define MCI_GETDEVCAPS_CAN_PLAY         0x00000008L
#define MCI_GETDEVCAPS_CAN_SAVE         0x00000009L

#define MCI_SYSINFO_QUANTITY            0x00000100L
#define MCI_SYSINFO_OPEN                0x00000200L
#define MCI_SYSINFO_NAME                0x00000400L
#define MCI_SYSINFO_INSTALLNAME         0x00000800L

#define MCI_SET_DOOR_OPEN               0x00000100L
#define MCI_SET_DOOR_CLOSED             0x00000200L
#define MCI_SET_TIME_FORMAT             0x00000400L
#define MCI_SET_AUDIO                   0x00000800L
#define MCI_SET_VIDEO                   0x00001000L
#define MCI_SET_ON                      0x00002000L
#define MCI_SET_OFF                     0x00004000L

#define MCI_SET_AUDIO_ALL               0x00000000L
#define MCI_SET_AUDIO_LEFT              0x00000001L
#define MCI_SET_AUDIO_RIGHT             0x00000002L

#define MCI_BREAK_KEY                   0x00000100L
#define MCI_BREAK_HWND                  0x00000200L
#define MCI_BREAK_OFF                   0x00000400L

#define MCI_RECORD_INSERT               0x00000100L
#define MCI_RECORD_OVERWRITE            0x00000200L

#define MCI_SOUND_NAME                  0x00000100L

#define MCI_SAVE_FILE                   0x00000100L  

#define MCI_LOAD_FILE                   0x00000100L  

// Data structure used by messages with no special parameters
typedef struct {
    DWORD   dwCallback;
} MCI_GENERIC_PARMS;
typedef MCI_GENERIC_PARMS FAR *LPMCI_GENERIC_PARMS;

typedef struct {
    DWORD   dwCallback;
    WORD    wDeviceID;       
    WORD    wReserved0;
    LPSTR   lpstrDeviceType; 
    LPSTR   lpstrElementName;
    LPSTR   lpstrAlias;      
} MCI_OPEN_PARMS;
typedef MCI_OPEN_PARMS FAR *LPMCI_OPEN_PARMS;

typedef struct {
    DWORD   dwCallback;
    DWORD   dwFrom;         
    DWORD   dwTo;           
} MCI_PLAY_PARMS;
typedef MCI_PLAY_PARMS FAR *LPMCI_PLAY_PARMS;

typedef struct {
    DWORD   dwCallback;
    DWORD   dwTo;           
} MCI_SEEK_PARMS;
typedef MCI_SEEK_PARMS FAR *LPMCI_SEEK_PARMS;

typedef struct {
    DWORD   dwCallback;
    DWORD   dwReturn;
    DWORD   dwItem;
    DWORD   dwTrack;
} MCI_STATUS_PARMS;
typedef MCI_STATUS_PARMS FAR * LPMCI_STATUS_PARMS;

typedef struct {
    DWORD   dwCallback;
    LPSTR   lpstrReturn;
    DWORD   dwRetSize;
} MCI_INFO_PARMS;
typedef MCI_INFO_PARMS FAR * LPMCI_INFO_PARMS;

typedef struct {
    DWORD   dwCallback;
    DWORD   dwReturn;
    DWORD   dwItem;
} MCI_GETDEVCAPS_PARMS;
typedef MCI_GETDEVCAPS_PARMS FAR * LPMCI_GETDEVCAPS_PARMS;

typedef struct {
    DWORD   dwCallback;    
    LPSTR   lpstrReturn;
    DWORD   dwRetSize;
    DWORD   dwNumber;
    WORD    wDeviceType;        
    WORD    wReserved0;
} MCI_SYSINFO_PARMS;
typedef MCI_SYSINFO_PARMS FAR * LPMCI_SYSINFO_PARMS;

typedef struct {
    DWORD   dwCallback;
    DWORD   dwTimeFormat;
    DWORD   dwAudio;
} MCI_SET_PARMS;
typedef MCI_SET_PARMS FAR *LPMCI_SET_PARMS;

typedef struct {
    DWORD   dwCallback;     
    int     nVirtKey;
    WORD    wReserved0;
    HWND    hwndBreak;
    WORD    wReserved1;
} MCI_BREAK_PARMS;
typedef MCI_BREAK_PARMS FAR * LPMCI_BREAK_PARMS;

typedef struct {
    DWORD   dwCallback;
    LPSTR   lpstrSoundName;
} MCI_SOUND_PARMS;
typedef MCI_SOUND_PARMS FAR * LPMCI_SOUND_PARMS;
    
typedef struct {
    DWORD   dwCallback;
    LPSTR   lpfilename;
} MCI_SAVE_PARMS;
typedef MCI_SAVE_PARMS FAR * LPMCI_SAVE_PARMS;

typedef struct {
    DWORD   dwCallback;
    LPSTR   lpfilename;
} MCI_LOAD_PARMS;
typedef MCI_LOAD_PARMS FAR * LPMCI_LOAD_PARMS;

typedef struct {
    DWORD   dwCallback;
    DWORD   dwFrom;         
    DWORD   dwTo;           
} MCI_RECORD_PARMS;
typedef MCI_RECORD_PARMS FAR *LPMCI_RECORD_PARMS;

/*****************************************************************************

    Standard command parameters for videodisc drivers

*****************************************************************************/

// string resource ID's for videodisc players 
// return ID's for videodisc status command

#define MCI_VD_MODE_PARK                (MCI_VD_OFFSET + 1)

// return ID's for videodisc MCI_GETDEVCAPS command

#define MCI_VD_MEDIA_CLV                (MCI_VD_OFFSET + 2)
#define MCI_VD_MEDIA_CAV                (MCI_VD_OFFSET + 3)
#define MCI_VD_MEDIA_OTHER              (MCI_VD_OFFSET + 4)
// Note: MCI_VD_OFFSET+5 is reserved

#define MCI_VD_FORMAT_TRACK             0x4001

#define MCI_VD_PLAY_REVERSE             0x00010000L  
#define MCI_VD_PLAY_FAST                0x00020000L  
#define MCI_VD_PLAY_SPEED               0x00040000L  
#define MCI_VD_PLAY_SCAN                0x00080000L  
#define MCI_VD_PLAY_SLOW                0x00100000L  

#define MCI_VD_SEEK_REVERSE             0x00010000L

#define MCI_VD_STATUS_SPEED             0x00004002L
#define MCI_VD_STATUS_FORWARD           0x00004003L
#define MCI_VD_STATUS_MEDIA_TYPE        0x00004004L
#define MCI_VD_STATUS_SIDE              0x00004005L
#define MCI_VD_STATUS_DISC_SIZE         0x00004006L

#define MCI_VD_GETDEVCAPS_CLV           0x00010000L
#define MCI_VD_GETDEVCAPS_CAV           0x00020000L

#define MCI_VD_SPIN_UP                  0x00010000L
#define MCI_VD_SPIN_DOWN                0x00020000L

#define MCI_VD_GETDEVCAPS_CAN_REVERSE   0x00004002L
#define MCI_VD_GETDEVCAPS_FAST_RATE     0x00004003L
#define MCI_VD_GETDEVCAPS_SLOW_RATE     0x00004004L
#define MCI_VD_GETDEVCAPS_NORMAL_RATE   0x00004005L

#define MCI_VD_STEP_FRAMES              0x00010000L
#define MCI_VD_STEP_REVERSE             0x00020000L

#define MCI_VD_ESCAPE_STRING            0x00000100L

typedef struct {
    DWORD   dwCallback;
    DWORD   dwFrom;         
    DWORD   dwTo;           
    DWORD   dwSpeed;         
    } MCI_VD_PLAY_PARMS;
typedef MCI_VD_PLAY_PARMS FAR *LPMCI_VD_PLAY_PARMS;

typedef struct {
    DWORD   dwCallback;
    DWORD   dwFrames;
} MCI_VD_STEP_PARMS;
typedef MCI_VD_STEP_PARMS FAR *LPMCI_VD_STEP_PARMS;

typedef struct {
    DWORD   dwCallback;
    LPSTR   lpstrCommand;           
} MCI_VD_ESCAPE_PARMS;
typedef MCI_VD_ESCAPE_PARMS FAR *LPMCI_VD_ESCAPE_PARMS;

/*****************************************************************************
  
    Standard command parameters for waveform audio drivers
  
*****************************************************************************/

// Note: MCI_WAVE_OFFSET+0 thru MCI_WAVE_OFFSET+1 are reserved

#define MCI_WAVE_OPEN_BUFFER            0x00010000L

#define MCI_WAVE_SET_FORMATTAG          0x00010000L
#define MCI_WAVE_SET_CHANNELS           0x00020000L
#define MCI_WAVE_SET_SAMPLESPERSEC      0x00040000L
#define MCI_WAVE_SET_AVGBYTESPERSEC     0x00080000L
#define MCI_WAVE_SET_BLOCKALIGN         0x00100000L
#define MCI_WAVE_SET_BITSPERSAMPLE      0x00200000L

#define MCI_WAVE_STATUS_FORMATTAG       0x00004001L
#define MCI_WAVE_STATUS_CHANNELS        0x00004002L
#define MCI_WAVE_STATUS_SAMPLESPERSEC   0x00004003L
#define MCI_WAVE_STATUS_AVGBYTESPERSEC  0x00004004L
#define MCI_WAVE_STATUS_BLOCKALIGN      0x00004005L
#define MCI_WAVE_STATUS_BITSPERSAMPLE   0x00004006L
#define MCI_WAVE_STATUS_LEVEL           0x00004007L

#define MCI_WAVE_INPUT                  0x00400000L
#define MCI_WAVE_OUTPUT                 0x00800000L


#define MCI_WAVE_SET_ANYINPUT           0x04000000L
#define MCI_WAVE_SET_ANYOUTPUT          0x08000000L

#define MCI_WAVE_GETDEVCAPS_INPUTS      0x00004001L
#define MCI_WAVE_GETDEVCAPS_OUTPUTS     0x00004002L

typedef struct {
    DWORD   dwCallback;
    WORD    wDeviceID;       
    WORD    wReserved0;
    LPSTR   lpstrDeviceType; 
    LPSTR   lpstrElementName;
    LPSTR   lpstrAlias;      
    DWORD   dwBufferSeconds;
} MCI_WAVE_OPEN_PARMS;
typedef MCI_WAVE_OPEN_PARMS FAR *LPMCI_WAVE_OPEN_PARMS;

typedef struct {
    DWORD   dwCallback;
    DWORD   dwFrom;         
    DWORD   dwTo;           
} MCI_WAVE_DELETE_PARMS;
typedef MCI_WAVE_DELETE_PARMS FAR *LPMCI_WAVE_DELETE_PARMS;


typedef struct {
    DWORD   dwCallback;
    DWORD   dwTimeFormat;
    DWORD   dwAudio;
    WORD    wInput;
    WORD    wReserved0;
    WORD    wOutput;
    WORD    wReserved1;
    WORD    wFormatTag;       
    WORD    wReserved2;
    WORD    nChannels;	      
    WORD    wReserved3;
    DWORD   nSamplesPerSec;   
    DWORD   nAvgBytesPerSec;  
    WORD    nBlockAlign;      
    WORD    wReserved4;
    WORD    wBitsPerSample;   
    WORD    wReserved5;
} MCI_WAVE_SET_PARMS;
typedef MCI_WAVE_SET_PARMS FAR * LPMCI_WAVE_SET_PARMS;

/*****************************************************************************

   Standard command parameters for CD audio drivers

*****************************************************************************/

// None

/*****************************************************************************

    Standard command parameters for Sequencer drivers

*****************************************************************************/

// string resource ID's for sequencers
// return ids for status division type

#define     MCI_SEQ_DIV_PPQN	        (0 + MCI_SEQ_OFFSET)
#define     MCI_SEQ_DIV_SMPTE_24        (1 + MCI_SEQ_OFFSET)
#define     MCI_SEQ_DIV_SMPTE_25        (2 + MCI_SEQ_OFFSET)
#define     MCI_SEQ_DIV_SMPTE_30DROP    (3 + MCI_SEQ_OFFSET)
#define     MCI_SEQ_DIV_SMPTE_30        (4 + MCI_SEQ_OFFSET)
// Note: MCI_SEQ_OFFSET+5 thru MCI_WAVE_OFFSET+11 are reserved

#define     MCI_SEQ_FORMAT_SONGPTR      0x4001
#define     MCI_SEQ_FILE    	        0x4002
#define     MCI_SEQ_MIDI                0x4003
#define     MCI_SEQ_SMPTE	        0x4004    

#define     MCI_SEQ_NONE		65533

#define MCI_SEQ_STATUS_TEMPO		0x00004002L
#define MCI_SEQ_STATUS_PORT		0x00004003L
#define MCI_SEQ_STATUS_SLAVE		0x00004007L
#define MCI_SEQ_STATUS_MASTER		0x00004008L
#define MCI_SEQ_STATUS_OFFSET		0x00004009L
#define MCI_SEQ_STATUS_DIVTYPE		0x0000400AL

#define MCI_SEQ_SET_TEMPO		0x00010000L
#define MCI_SEQ_SET_PORT		0x00020000L
#define MCI_SEQ_SET_SLAVE        	0x00040000L
#define MCI_SEQ_SET_MASTER        	0x00080000L
#define MCI_SEQ_SET_OFFSET		0x01000000L

typedef struct {
    DWORD   dwCallback;
    DWORD   dwTimeFormat;
    DWORD   dwAudio;
    DWORD   dwTempo;
    DWORD   dwPort;
    DWORD   dwSlave;
    DWORD   dwMaster;
    DWORD   dwOffset;
} MCI_SEQ_SET_PARMS;
typedef MCI_SEQ_SET_PARMS FAR * LPMCI_SEQ_SET_PARMS;

/*****************************************************************************

    Standard command parameters for animation

*****************************************************************************/


#define MCI_ANIM_OPEN_WS                0x00010000L  // MCI_OPEN 
#define MCI_ANIM_OPEN_PARENT            0x00020000L  
#define MCI_ANIM_OPEN_NOSTATIC          0x00040000L  

#define MCI_ANIM_PLAY_SPEED             0x00010000L  // MCI_PLAY 
#define MCI_ANIM_PLAY_REVERSE           0x00020000L  
#define MCI_ANIM_PLAY_FAST              0x00040000L  
#define MCI_ANIM_PLAY_SLOW              0x00080000L  
#define MCI_ANIM_PLAY_SCAN              0x00100000L  

#define MCI_ANIM_STEP_REVERSE           0x00010000L  // MCI_STEP
#define MCI_ANIM_STEP_FRAMES            0x00020000L

#define MCI_ANIM_STATUS_SPEED           0x00004001L  // MCI_STATUS 
#define MCI_ANIM_STATUS_FORWARD         0x00004002L
#define MCI_ANIM_STATUS_HWND            0x00004003L
#define MCI_ANIM_STATUS_HPAL            0x00004004L
#define MCI_ANIM_STATUS_STRETCH         0x00004005L

#define MCI_ANIM_INFO_TEXT              0x00010000L  // MCI_INFO

#define MCI_ANIM_GETDEVCAPS_CAN_REVERSE 0x00004001L  // MCI_GETDEVCAPS
#define MCI_ANIM_GETDEVCAPS_FAST_RATE   0x00004002L
#define MCI_ANIM_GETDEVCAPS_SLOW_RATE   0x00004003L
#define MCI_ANIM_GETDEVCAPS_NORMAL_RATE 0x00004004L
#define MCI_ANIM_GETDEVCAPS_PALETTES    0x00004006L
#define MCI_ANIM_GETDEVCAPS_CAN_STRETCH 0x00004007L
#define MCI_ANIM_GETDEVCAPS_MAX_WINDOWS 0x00004008L

#define MCI_ANIM_REALIZE_NORM           0x00010000L  // MCI_REALIZE
#define MCI_ANIM_REALIZE_BKGD           0x00020000L

#define MCI_ANIM_WINDOW_HWND            0x00010000L  // MCI_WINDOW
#define MCI_ANIM_WINDOW_DEFAULT         0x00020000L
#define MCI_ANIM_WINDOW_STATE           0x00040000L
#define MCI_ANIM_WINDOW_TEXT            0x00080000L
#define MCI_ANIM_WINDOW_ENABLE_STRETCH  0x00100000L
#define MCI_ANIM_WINDOW_DISABLE_STRETCH 0x00200000L

#define MCI_ANIM_RECT                   0x00010000L  // MCI_SOURCE,MCI_DESTINATION, MCI_UPDATE

#define MCI_ANIM_PUT_SOURCE             0x00020000L  // MCI_PUT
#define MCI_ANIM_PUT_DESTINATION        0x00040000L 

#define MCI_ANIM_WHERE_SOURCE           0x00020000L  // MCI_WHERE
#define MCI_ANIM_WHERE_DESTINATION      0x00040000L 

#define MCI_ANIM_UPDATE_HDC             0x00020000L  // MCI_UPDATE

typedef struct {
    DWORD   dwCallback;
    WORD    wDeviceID;       
    WORD    wReserved0;
    LPSTR   lpstrDeviceType; 
    LPSTR   lpstrElementName;
    LPSTR   lpstrAlias;      
    DWORD   dwStyle;
    WORD    hWndParent;          
    WORD    wReserved1;
} MCI_ANIM_OPEN_PARMS;
typedef MCI_ANIM_OPEN_PARMS FAR *LPMCI_ANIM_OPEN_PARMS;

typedef struct {
    DWORD   dwCallback;
    DWORD   dwFrom;         
    DWORD   dwTo;           
    DWORD   dwSpeed;          
} MCI_ANIM_PLAY_PARMS;
typedef MCI_ANIM_PLAY_PARMS FAR *LPMCI_ANIM_PLAY_PARMS;

typedef struct {
    DWORD   dwCallback;
    DWORD   dwFrames;
} MCI_ANIM_STEP_PARMS;
typedef MCI_ANIM_STEP_PARMS FAR *LPMCI_ANIM_STEP_PARMS;

typedef struct {
    DWORD   dwCallback;
    WORD    hWnd;          
    WORD    wReserved1;
    WORD    nCmdShow;          
    WORD    wReserved2;
    LPSTR   lpstrText;
} MCI_ANIM_WINDOW_PARMS;
typedef MCI_ANIM_WINDOW_PARMS FAR * LPMCI_ANIM_WINDOW_PARMS;

typedef struct {
    DWORD   dwCallback;
    RECT    rc;
} MCI_ANIM_RECT_PARMS;
typedef MCI_ANIM_RECT_PARMS FAR * LPMCI_ANIM_RECT_PARMS;

typedef struct {
    DWORD   dwCallback;
    RECT    rc;
    HDC     hDC;
} MCI_ANIM_UPDATE_PARMS;
typedef MCI_ANIM_UPDATE_PARMS FAR * LPMCI_ANIM_UPDATE_PARMS;


/*****************************************************************************

    Standard command parameters for overlay

*****************************************************************************/


#define MCI_OVLY_OPEN_WS                0x00010000L  // MCI_OPEN 
#define MCI_OVLY_OPEN_PARENT            0x00020000L  

#define MCI_OVLY_STATUS_HWND            0x00004001L
#define MCI_OVLY_STATUS_STRETCH         0x00004002L

#define MCI_OVLY_INFO_TEXT              0x00010000L  // MCI_INFO

#define MCI_OVLY_GETDEVCAPS_CAN_STRETCH 0x00004001L
#define MCI_OVLY_GETDEVCAPS_CAN_FREEZE  0x00004002L
#define MCI_OVLY_GETDEVCAPS_MAX_WINDOWS 0x00004003L

#define MCI_OVLY_WINDOW_HWND            0x00010000L  // MCI_WINDOW
#define MCI_OVLY_WINDOW_DEFAULT         0x00020000L
#define MCI_OVLY_WINDOW_STATE           0x00040000L
#define MCI_OVLY_WINDOW_TEXT            0x00080000L
#define MCI_OVLY_WINDOW_ENABLE_STRETCH  0x00100000L
#define MCI_OVLY_WINDOW_DISABLE_STRETCH 0x00200000L

#define MCI_OVLY_RECT                   0x00010000L  // MCI_WHERE, MCI_UPDATE, MCI_FREEZE, MCI_UNFREEZE

#define MCI_OVLY_PUT_SOURCE             0x00020000L  // MCI_PUT
#define MCI_OVLY_PUT_DESTINATION        0x00040000L 
#define MCI_OVLY_PUT_FRAME              0x00080000L 

#define MCI_OVLY_WHERE_SOURCE           0x00020000L  // MCI_WHERE
#define MCI_OVLY_WHERE_DESTINATION      0x00040000L 
#define MCI_OVLY_WHERE_FRAME            0x00080000L 

typedef struct {
    DWORD   dwCallback;
    WORD    wDeviceID;       
    WORD    wReserved0;
    LPSTR   lpstrDeviceType; 
    LPSTR   lpstrElementName;
    LPSTR   lpstrAlias;      
    DWORD   dwStyle;
    WORD    hWndParent;          
    WORD    wReserved1;
 } MCI_OVLY_OPEN_PARMS;
typedef MCI_OVLY_OPEN_PARMS FAR *LPMCI_OVLY_OPEN_PARMS;

typedef struct {
    DWORD   dwCallback;
    WORD    hWnd;          
    WORD    wReserved1;
    WORD    nCmdShow;          
    WORD    wReserved2;
    LPSTR   lpstrText;
} MCI_OVLY_WINDOW_PARMS;
typedef MCI_OVLY_WINDOW_PARMS FAR * LPMCI_OVLY_WINDOW_PARMS;

typedef struct {
    DWORD   dwCallback;
    RECT    rc;
} MCI_OVLY_RECT_PARMS;
typedef MCI_OVLY_RECT_PARMS FAR * LPMCI_OVLY_RECT_PARMS;

 
/*****************************************************************************

    MCI support functions

*****************************************************************************/

typedef int (FAR PASCAL *YIELDPROC) (WORD wDeviceID, DWORD dwYieldData);

extern DWORD FAR PASCAL mciSendCommand (WORD wDeviceID, WORD wMessage,
                                       DWORD dwParam1, DWORD dwParam2);

extern DWORD FAR PASCAL mciSendString (LPSTR lpstrCommand,
                                      LPSTR lpstrReturnString,
                                      WORD wReturnLength,
                                      HANDLE hCallback);

extern WORD FAR PASCAL mciGetDeviceID (LPSTR lpstrName);
extern WORD FAR PASCAL mciGetDeviceIDFromElementID (DWORD dwElementID,
                                                    LPSTR lpstrType);

extern WORD FAR PASCAL mciGetErrorString (DWORD wError, LPSTR lpstrBuffer,
                                          WORD wLength);
extern BOOL FAR PASCAL mciExecute (LPSTR lpstrCommand);

extern WORD mciSetYieldProc (WORD wDeviceID, YIELDPROC fpYieldProc,
                             DWORD dwYieldData);
#endif		// ifndef MMNOMCI

/****************************************************************************

    DISPLAY Driver extensions

****************************************************************************/

#ifndef C1_TRANSPARENT
    #define CAPS1           94          // other caps
    #define C1_TRANSPARENT  0x0001      // new raster cap
    #define NEWTRANSPARENT  3           // use with SetBkMode()

    #define QUERYROPSUPPORT 40          // use to determine ROP support
#endif

/****************************************************************************

    DIB Driver extensions

****************************************************************************/

#define SELECTDIB       41              // DIB.DRV select dib excape
#define DIBINDEX(n)     MAKELONG((n),0x10FF)

/****************************************************************************

    ScreenSave stuff

    the current application will recive a syscommand of SC_SCREENSAVE just
    before the screen saver is invoked.  If the app wishes to prevent a
    screen save, return non-0 otherwise pass to DefWindowProc()

****************************************************************************/

#ifndef SC_SCREENSAVE

    #define SC_SCREENSAVE   0xF140

#endif

#ifndef NOMMIO
/* mmio.h
 *
 * Multimedia File I/O Library.
 */

typedef DWORD		FOURCC;		// a four character code
typedef char huge *	HPSTR;		// a huge version of LPSTR
typedef HANDLE		HMMIO;		// a handle to an open file

#define CFSEPCHAR	'+'		// compound file name separator char.

typedef LONG (FAR PASCAL MMIOPROC)(LPSTR lpmmioinfo, WORD wMsg,
	LONG lParam1, LONG lParam2);
typedef MMIOPROC FAR *LPMMIOPROC;

typedef struct _MMIOINFO	// structure for representing MMIO state
{
	/* general fields */
	DWORD		dwFlags;	// general status flags
	FOURCC		fccIOProc;	// ptr. to I/O procedure
	LPMMIOPROC	pIOProc;	// ptr. to I/O procedure
	WORD		wErrorRet;	// place for error to be returned
	WORD		wReserved;	// <for structure alignment>
	    
	/* fields maintained by MMIO functions during buffered I/O */
	LONG		cchBuffer;	// size of I/O buffer (or 0L)
	HPSTR		pchBuffer;	// start of I/O buffer (or NULL)
	HPSTR		pchNext;	// ptr. to next byte to read/write
	HPSTR		pchEndRead;	// ptr. to last valid byte to read
	HPSTR		pchEndWrite;	// ptr. to last available byte to write
	LONG		lBufOffset;	// disk offset of start of buffer

	/* fields maintained by I/O procedure */
	LONG		lDiskOffset;	// disk offset of next read or write
	DWORD		adwInfo[3];	// data specific to type of MMIOPROC

	/* other fields maintained by MMIO */
	DWORD		dwReserved1;	// reserved for MMIO use
	DWORD		dwReserved2;	// reserved for MMIO use
	HMMIO		hmmio;		// handle to open file
} MMIOINFO, FAR * LPMMIOINFO;

typedef struct _MMCKINFO	// structure for representing RIFF chunk info.
{
	FOURCC		ckid;		// chunk ID
	DWORD		cksize;		// chunk size
	FOURCC		fccType;	// form type or list type
	DWORD		dwDataOffset;	// offset of data portion of chunk
	DWORD		dwFlags;	// flags used by MMIO functions
} MMCKINFO, FAR *LPMMCKINFO;

/* <dwFlags> field of MMIOINFO structure -- many same as OpenFile() flags */
/* Low word of flags will be passed to OpenFile() -- therefore, any MMIO-
    specific flags should be in the high word.	*/
#define MMIO_RWMODE	0x00000003	// open file for reading/writing/both
#define MMIO_SHAREMODE	0x00000070	// file sharing mode number
#define MMIO_CREATE	0x00001000	// create new file (or truncate file)
#define MMIO_DELETE	0x00000200	// create new file (or truncate file)
#define MMIO_ALLOCBUF	0x00010000	// mmioOpen() should allocate a buffer
#define MMIO_DIRTY	0x10000000	// I/O buffer is dirty
/* MMIO_DIRTY is also used in the <dwFlags> field of MMCKINFO structure */

/* read/write mode numbers (bit field MMIO_RWMODE) */
#define MMIO_READ	0x00000000	// open file for reading only
#define MMIO_WRITE	0x00000001	// open file for writing only
#define MMIO_READWRITE	0x00000002	// open file for reading and writing

/* share mode numbers (bit field MMIO_SHAREMODE) */
#define MMIO_COMPAT	0x00000000	// compatibility mode
#define MMIO_EXCLUSIVE	0x00000010	// exclusive-access mode
#define MMIO_DENYWRITE	0x00000020	// deny writing to other processes
#define MMIO_DENYREAD	0x00000030	// deny reading to other processes
#define MMIO_DENYNONE	0x00000040	// deny nothing to other processes

/* flags for other functions */
#define MMIO_FHOPEN		0x0010	// mmioClose(): keep file handle open
#define MMIO_EMPTYBUF		0x0010	// mmioFlush(): empty the I/O buffer
#define MMIO_TOUPPER		0x0010	// mmioStringToFOURCC(): cvt. to u-case
#define MMIO_INSTALLPROC    0x00010000	// mmioInstallIOProc(): install MMIOProc
#define MMIO_REMOVEPROC	    0x00020000	// mmioInstallIOProc(): remove MMIOProc
#define MMIO_FINDPROC	    0x00040000	// mmioInstallIOProc(): find an MMIOProc
#define MMIO_FINDCHUNK		0x0010	// mmioDescend(): find a chunk by ID
#define MMIO_FINDRIFF		0x0020	// mmioDescend(): find a LIST chunk
#define MMIO_FINDLIST		0x0040	// mmioDescend(): find a RIFF chunk
#define MMIO_CREATERIFF		0x0020	// mmioCreateChunk(): make a LIST chunk
#define MMIO_CREATELIST		0x0040	// mmioCreateChunk(): make a RIFF chunk

/* message numbers for MMIOPROC */
#define MMIOM_READ      MMIO_READ	// read (must equal MMIO_READ!)
#define MMIOM_WRITE    MMIO_WRITE	// write (must equal MMIO_WRITE!)
#define MMIOM_SEEK		2	// seek to a new position in file
#define MMIOM_OPEN		3	// open file
#define MMIOM_CLOSE		4	// close file
#define MMIOM_WRITEFLUSH	5	// write and flush
#define MMIOM_USER	   0x8000	// beginning of user-defined messages

/* standard four character codes */
#define FOURCC_RIFF	mmioFOURCC('R', 'I', 'F', 'F')
#define FOURCC_LIST	mmioFOURCC('L', 'I', 'S', 'T')

/* four character codes used to identify standard built-in I/O procedures */
#define FOURCC_DOS	mmioFOURCC('D', 'O', 'S', ' ')
#define FOURCC_MEM	mmioFOURCC('M', 'E', 'M', ' ')

/* error numbers */
#define MMIOERR_BASE		256
#define MMIOERR_FILENOTFOUND	(MMIOERR_BASE + 1)  // file not found
#define MMIOERR_OUTOFMEMORY	(MMIOERR_BASE + 2)  // out of memory
#define MMIOERR_CANNOTOPEN	(MMIOERR_BASE + 3)  // cannot open
#define MMIOERR_CANNOTCLOSE	(MMIOERR_BASE + 4)  // cannot close
#define MMIOERR_CANNOTREAD	(MMIOERR_BASE + 5)  // cannot read
#define MMIOERR_CANNOTWRITE	(MMIOERR_BASE + 6)  // cannot write
#define MMIOERR_CANNOTSEEK	(MMIOERR_BASE + 7)  // cannot seek
#define MMIOERR_CANNOTEXPAND	(MMIOERR_BASE + 8)  // cannot expand file
#define MMIOERR_CHUNKNOTFOUND	(MMIOERR_BASE + 9)  // chunk not found
#define MMIOERR_UNBUFFERED	(MMIOERR_BASE + 10) // file is unbuffered

/* flags for mmioSeek() */
#ifndef SEEK_SET
#define SEEK_SET	0		// seek to an absolute position
#define SEEK_CUR	1		// seek relative to current position
#define SEEK_END	2		// seek relative to end of file
#endif

/* other constants */
#define MMIO_DEFAULTBUFFER	8192	// default buffer size

/* macros */
#define mmioFOURCC( ch0, ch1, ch2, ch3 )				\
		( (DWORD)(BYTE)(ch0) | ( (DWORD)(BYTE)(ch1) << 8 ) |	\
		( (DWORD)(BYTE)(ch2) << 16 ) | ( (DWORD)(BYTE)(ch3) << 24 ) )

/* MMIO prototypes */
FOURCC FAR PASCAL mmioStringToFOURCC(LPSTR sz, WORD wFlags);
LPMMIOPROC FAR PASCAL mmioInstallIOProc(FOURCC fccIOProc, LPMMIOPROC pIOProc,
	DWORD dwFlags);
HMMIO FAR PASCAL mmioOpen(LPSTR szFileName, LPMMIOINFO lpmmioinfo,
	DWORD dwOpenFlags);
WORD FAR PASCAL mmioClose(HMMIO hmmio, WORD wFlags);
LONG FAR PASCAL mmioRead(HMMIO hmmio, HPSTR pch, LONG cch);
LONG FAR PASCAL mmioWrite(HMMIO hmmio, HPSTR pch, LONG cch);
LONG FAR PASCAL mmioSeek(HMMIO hmmio, LONG lOffset, int iOrigin);
WORD FAR PASCAL mmioGetInfo(HMMIO hmmio, LPMMIOINFO lpmmioinfo, WORD wFlags);
WORD FAR PASCAL mmioSetInfo(HMMIO hmmio, LPMMIOINFO lpmmioinfo, WORD wFlags);
WORD FAR PASCAL mmioSetBuffer(HMMIO hmmio, LPSTR pchBuffer, LONG cchBuffer,
	WORD wFlags);
WORD FAR PASCAL mmioFlush(HMMIO hmmio, WORD wFlags);
WORD FAR PASCAL mmioAdvance(HMMIO hmmio, LPMMIOINFO lpmmioinfo, WORD wFlags);
LONG FAR PASCAL mmioSendMessage(HANDLE hmmio, WORD wMsg,
	LONG lParam1, LONG lParam2);

/* RIFF I/O prototypes */
WORD FAR PASCAL mmioDescend(HANDLE hmmio, LPMMCKINFO lpck,
	LPMMCKINFO lpckParent, WORD wFlags);
WORD FAR PASCAL mmioAscend(HANDLE hmmio, LPMMCKINFO lpck, WORD wFlags);
WORD FAR PASCAL mmioCreateChunk(HANDLE hmmio, LPMMCKINFO lpck, WORD wFlags);

#endif /* NOMMIO */
