//     (C) Copyright Microsoft Corp. 1991.  All rights reserved.
//
//     You have a royalty-free right to use, modify, reproduce and 
//     distribute the Sample Files (and/or any modified version) in 
//     any way you find useful, provided that you agree that 
//     Microsoft has no warranty obligations or liability for any 
//     Sample Application Files which are modified. 

#include <windows.h>
#include <mmsystem.h>
#include "joytoy.h"

BOOL InitJoyToy(HANDLE hInstance);

char szAppName[] = "JoyToy";     
HANDLE hButton1, hButton2;
LPSTR lpButton1, lpButton2;

int PASCAL WinMain(hInstance,hPrevInstance,lpszCmdLine,cmdShow)
HANDLE hInstance,hPrevInstance;
LPSTR lpszCmdLine;
int cmdShow;
{
    MSG msg;

    if(! InitJoyToy(hInstance))
        return FALSE;

    while(GetMessage(&msg, NULL, 0, 0))
    {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

    UnlockResource(hButton1);
    UnlockResource(hButton2);

    FreeResource(hButton1);
    FreeResource(hButton2);
    
    return (msg.wParam);
}

void DrawSight(DWORD lParam)
{
    WORD x, y;
    HDC hDC;
    POINT pt;
    
    GetCursorPos(&pt);

    x = LOWORD(lParam) >> 11;
    y = HIWORD(lParam) >> 11;

    if(x <= 12)
        pt.x = pt.x + x - 17;
    else if(x >= 20)
        pt.x = pt.x + x - 15;

    if(y <= 12)
        pt.y = pt.y + y - 17;
    else if(y >= 20)
        pt.y = pt.y + y - 15;

    SetCursorPos(pt.x, pt.y);
}

void DrawFire(HWND hWnd)
{
    HDC hDC;
    POINT pt;
    int i;

    GetCursorPos(&pt);
    hDC = GetDC(hWnd);
    PatBlt(hDC, pt.x - 2, pt.y - 2, 4, 4, BLACKNESS);
    ReleaseDC(hWnd, hDC);
}


long FAR PASCAL MainWndProc(HWND hWnd, unsigned message, WORD wParam, LONG lParam)
{
    switch(message)
    {
        case WM_CREATE:
            if(joySetCapture(hWnd, JOYSTICKID1, NULL, FALSE))
            {
                MessageBeep(MB_ICONEXCLAMATION);
                MessageBox(hWnd, "Couldn't capture the joystick", NULL, MB_OK | MB_ICONEXCLAMATION);
                PostMessage(hWnd, WM_CLOSE, 0, 0L);
            }            
            break;

        case WM_ACTIVATE:
        case WM_ACTIVATEAPP:
            if(wParam)
                break;

        case WM_LBUTTONDOWN:
        case WM_MBUTTONDOWN:
        case WM_RBUTTONDOWN:
        case WM_KEYDOWN:
        case WM_CHAR:
            PostMessage(hWnd,WM_CLOSE,0,0L);
            break;

        case WM_ERASEBKGND:
            return 0l;
 
        case MM_JOY1BUTTONDOWN :
            if (wParam & JOY_BUTTON1)
            {
                sndPlaySound(lpButton1, SND_LOOP | SND_ASYNC | SND_MEMORY);
                DrawFire(hWnd);
            }
            else if (wParam & JOY_BUTTON2)
            {
                sndPlaySound(lpButton2, SND_ASYNC | SND_MEMORY |  SND_LOOP);
                DrawFire(hWnd);
            }
            break;

        case MM_JOY1BUTTONUP :
            sndPlaySound(NULL, 0);
            break;

        case MM_JOY1MOVE :
            if(wParam & (JOY_BUTTON1 | JOY_BUTTON2))
                DrawFire(hWnd);
            DrawSight(lParam);
            break;

        case WM_QUERYOPEN:
            return 0l;
    
        case WM_DESTROY:
            joyReleaseCapture(JOYSTICKID1);
            sndPlaySound(NULL, 0);
            PostQuitMessage(0);
            break;
    }
    return DefWindowProc(hWnd, message, wParam, lParam);
}



BOOL InitJoyToy(HANDLE hInstance)
{
    WNDCLASS wc;
    HWND     hWnd;
    char     buf[80];
    
    if (!joyGetNumDevs())
        return FALSE;
   
    hButton1 = LoadResource(hInstance, FindResource(hInstance, "BUTTON1", "WAVE"));
    hButton2 = LoadResource(hInstance, FindResource(hInstance, "BUTTON2", "WAVE"));
    lpButton1 = LockResource(hButton1);
    lpButton2 = LockResource(hButton2);

    wc.lpszClassName    = szAppName;
    wc.style            = CS_HREDRAW | CS_VREDRAW;
    wc.hCursor          = LoadCursor(NULL, IDC_CROSS);
    wc.hIcon            = NULL;
    wc.lpszMenuName     = NULL;
    wc.hbrBackground    = GetStockObject(BLACK_BRUSH);
    wc.hInstance        = hInstance;
    wc.lpfnWndProc      = MainWndProc;
    wc.cbClsExtra       = 0;
    wc.cbWndExtra       = 0;
    
    if(! RegisterClass(&wc))
        return FALSE;   

    hWnd = CreateWindow(szAppName, szAppName, WS_POPUP | WS_VISIBLE, 0, 0,
        GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN),
        NULL, NULL, hInstance, NULL);

    return (hWnd != NULL);
}

